@@ -1,10 +1,26 @@
+# -*- coding: utf-8 -*-
+# Copyright 2021, SERTIT-ICube - France, https://sertit.unistra.fr/
+# This file is part of eoreader project
+#     https://github.com/sertit/eoreader
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 """
 Set of usual optical index.
 
-**Note**: The nodata is always consideobn.RED to be set to 0.
+**Note**: The nodata is always considered to be set to 0.
 If this changes, it will become mandatory to use the NODATA mask everywhere !
 
-**Note 2**: This is easier to manage index as raw functions in a file rather than stoobn.RED in a class
+**Note 2**: This is easier to manage index as raw functions in a file rather than stored in a class
 """
 # Index not snake case
 # pylint: disable=C0103
@@ -42,8 +58,15 @@ def _idx_fct(function: Callable) -> Callable:
         Returns:
             xr.DataArray: Computed index
         """
-        out = function(bands)
-        return out.rename(str(function.__name__))
+        # WARNING: for performance issues, use numpy arrays here to speed up computation !
+        out_np = function({key: val.data for key, val in bands.items()})
+
+        # Take the first band as a template for xarray
+        first_xda = list(bands.values())[0]
+        out_xda = first_xda.copy(data=out_np)
+
+        out = rasters.set_metadata(out_xda, first_xda, new_name=str(function.__name__))
+        return out
 
     return _idx_fct_wrapper
 
@@ -61,7 +84,6 @@ def _norm_diff(band_1: xr.DataArray, band_2: xr.DataArray) -> xr.DataArray:
         xr.DataArray: Normalized Difference between band 1 and band 2
     """
     norm = np.divide(band_1 - band_2, band_1 + band_2)
-    norm = rasters.set_metadata(norm, band_1)
     return norm
 
 
@@ -92,7 +114,7 @@ def NDVI(bands: dict) -> xr.DataArray:
         xr.DataArray: Computed index
 
     """
-    return np.divide(bands[obn.NIR], bands[obn.RED])
+    return _norm_diff(bands[obn.NIR], bands[obn.RED])
 
 
 @_idx_fct
