@@ -1,3 +1,19 @@
+# -*- coding: utf-8 -*-
+# Copyright 2021, SERTIT-ICube - France, https://sertit.unistra.fr/
+# This file is part of eoreader project
+#     https://github.com/sertit/eoreader
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 """ Landsat products """
 import glob
 import logging
@@ -20,7 +36,7 @@ from eoreader.bands.bands import OpticalBandNames as obn
 from eoreader.exceptions import InvalidProductError, InvalidTypeError
 from eoreader.products.optical.optical_product import OpticalProduct
 from eoreader.utils import DATETIME_FMT, EOREADER_NAME
-from sertit import files, rasters
+from sertit import files, rasters, rasters_rio
 from sertit.misc import ListEnum
 from sertit.rasters import XDS_TYPE
 
@@ -147,13 +163,12 @@ class LandsatProduct(OpticalProduct):
         """
         nodata_band = self._get_path(self._nodata_band_id)
 
-        # Vectorize the nodata band
-        nodata = rasters.vectorize(nodata_band, values=1)
+        # Vectorize the nodata band (rasters_rio is faster)
+        footprint = rasters_rio.vectorize(
+            nodata_band, values=1, keep_values=False, dissolve=True
+        ).convex_hull
 
-        # Clip the extent with the nodata
-        footprint = gpd.overlay(self.extent(), nodata, how="symmetric_difference")
-
-        return footprint
+        return gpd.GeoDataFrame(geometry=footprint.geometry, crs=footprint.crs)
 
     def _get_tile_name(self) -> str:
         """
@@ -386,7 +401,7 @@ class LandsatProduct(OpticalProduct):
         if as_pd:
             mtd_name = f"{self.name}_MTL.txt"
             if self.is_archived:
-                # We need to extract the file in memry to be used with pandas
+                # We need to extract the file in memory to be used with pandas
                 tar_ds = tarfile.open(self.path, "r")
                 info = [f.name for f in tar_ds.getmembers() if mtd_name in f.name][0]
                 mtd_path = tar_ds.extractfile(info)
