@@ -1,3 +1,19 @@
+# -*- coding: utf-8 -*-
+# Copyright 2021, SERTIT-ICube - France, https://sertit.unistra.fr/
+# This file is part of eoreader project
+#     https://github.com/sertit/eoreader
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 """ Super class for optical products """
 
 import logging
@@ -8,8 +24,7 @@ from typing import Union
 import geopandas as gpd
 import numpy as np
 import rasterio
-import xarray as xr
-from rasterio import crs as rcrs
+from rasterio import crs as riocrs
 from rasterio.enums import Resampling
 
 from eoreader.bands import index
@@ -80,7 +95,7 @@ class OpticalProduct(Product):
         default_band = self.get_default_band()
         return self.get_band_paths([default_band])[default_band]
 
-    def crs(self) -> rcrs.CRS:
+    def crs(self) -> riocrs.CRS:
         """
         Get UTM projection of the tile
 
@@ -245,9 +260,7 @@ class OpticalProduct(Product):
             mask = np.expand_dims(mask, axis=0)
 
         # Set masked values to nodata
-        band_arr = xr.where(mask, self.nodata, band_arr)
-
-        return rasters.set_nodata(band_arr, self.nodata)
+        return band_arr.where(mask == 0, np.nan)
 
     def _load(
         self, bands: list, resolution: float = None, size: Union[list, tuple] = None
@@ -430,7 +443,6 @@ class OpticalProduct(Product):
             XDS_TYPE: Mask as xarray
         """
         mask = xds.copy(data=np.where(cond, self._mask_true, self._mask_false))
-        mask = xr.where(nodata, self.nodata, mask)
-        mask = rasters.set_nodata(mask, self.nodata)
+        mask = mask.where(nodata == 0, np.nan)
 
         return mask
