@@ -1,3 +1,19 @@
+# -*- coding: utf-8 -*-
+# Copyright 2021, SERTIT-ICube - France, https://sertit.unistra.fr/
+# This file is part of eoreader project
+#     https://github.com/sertit/eoreader
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 """ Sentinel-3 products """
 import logging
 import os
@@ -25,7 +41,7 @@ from eoreader.env_vars import S3_DEF_RES
 from eoreader.exceptions import InvalidProductError, InvalidTypeError
 from eoreader.products.optical.optical_product import OpticalProduct
 from eoreader.utils import DATETIME_FMT, EOREADER_NAME
-from sertit import files, misc, rasters, snap, strings, vectors
+from sertit import files, misc, rasters, rasters_rio, snap, strings, vectors
 from sertit.misc import ListEnum
 from sertit.rasters import XDS_TYPE
 
@@ -169,6 +185,26 @@ class S3Product(OpticalProduct):
         else:
             raise InvalidProductError(f"Invalid Sentinel-3 name: {self.name}")
 
+    def footprint(self) -> gpd.GeoDataFrame:
+        """
+        Get UTM footprint of the products (without nodata, *in french == emprise utile*)
+
+        ```python
+        >>> from eoreader.reader import Reader
+        >>> path = r"S2A_MSIL1C_20200824T110631_N0209_R137_T30TTK_20200824T150432.SAFE.zip"
+        >>> prod = Reader().open(path)
+        >>> prod.footprint()
+           index                                           geometry
+        0      0  POLYGON ((199980.000 4500000.000, 199980.000 4...
+        ```
+
+        Returns:
+            gpd.GeoDataFrame: Footprint as a GeoDataFrame
+        """
+        return rasters.get_footprint(
+            self.get_default_band_path()
+        )  # Processed by SNAP: the nodata is set
+
     def get_datetime(self, as_datetime: bool = False) -> Union[str, datetime]:
         """
         Get the product's acquisition datetime, with format `YYYYMMDDTHHMMSS` <-> `%Y%m%dT%H%M%S`
@@ -504,17 +540,19 @@ class S3Product(OpticalProduct):
             return band_arr
 
         # Open flag file
-        qual_arr = rasters.read(
+        qual_arr, _ = rasters_rio.read(
             qual_flags_path,
             resolution=resolution,
             size=size,
             resampling=Resampling.nearest,  # Nearest to keep the flags
             masked=False,
-        ).astype(np.uint32)
-        invalid, sat = rasters.read_bit_array(qual_arr, [invalid_id, sat_band_id])
+        )
+        invalid, sat = rasters.read_bit_array(
+            qual_arr.astype(np.uint32), [invalid_id, sat_band_id]
+        )
 
         # Get nodata mask
-        no_data = np.where(np.isnan(band_arr.data), nodata_true, nodata_false)
+        no_data = np.where(np.isnan(band_arr), nodata_true, nodata_false)
 
         # Combine masks
         mask = no_data | invalid | sat
@@ -561,7 +599,7 @@ class S3Product(OpticalProduct):
             return band_arr
 
         # Open flag file
-        qual_arr = rasters.read(
+        qual_arr, _ = rasters_rio.read(
             qual_flags_path,
             resolution=resolution,
             size=size,
@@ -570,7 +608,7 @@ class S3Product(OpticalProduct):
         )
 
         # Set no data for everything (except ISP) that caused an exception
-        exception = np.where(qual_arr.data > 2, nodata_true, nodata_false)
+        exception = np.where(qual_arr > 2, nodata_true, nodata_false)
 
         # Get nodata mask
         no_data = np.where(np.isnan(band_arr.data), nodata_true, nodata_false)
@@ -642,9 +680,7 @@ class S3Product(OpticalProduct):
 
                 # Convert to geotiffs and set no data with only keeping the first band
                 arr = rasters.read(rasters.get_dim_img_path(out_dim, snap_band_name))
-                arr = rasters.set_nodata(
-                    xr.where(arr == self._snap_no_data, self.nodata, arr), self.nodata
-                )
+                arr = arr.where(arr != self._snap_no_data, np.nan)
                 rasters.write(arr, out_tif, dtype=np.float32)
 
         # Get the wanted bands (not the quality flags here !)
@@ -983,7 +1019,7 @@ class S3Product(OpticalProduct):
                 size=size,
                 resampling=Resampling.nearest,
                 masked=False,
-            ).astype(np.uint16)
+            )
 
             # Get nodata mask
             # nodata = np.where(np.isnan(clouds_array), 1, 0)
