@@ -1,3 +1,19 @@
+# -*- coding: utf-8 -*-
+# Copyright 2021, SERTIT-ICube - France, https://sertit.unistra.fr/
+# This file is part of eoreader project
+#     https://github.com/sertit/eoreader
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 """ Product, superclass of all EOReader satellites products """
 # pylint: disable=W0107
 from __future__ import annotations
@@ -15,7 +31,7 @@ import geopandas as gpd
 import numpy as np
 import rasterio
 import xarray as xr
-from rasterio import crs as rcrs
+from rasterio import crs as riocrs
 from rasterio import warp
 from rasterio.enums import Resampling
 
@@ -224,7 +240,10 @@ class Product:
         Returns:
             gpd.GeoDataFrame: Footprint as a GeoDataFrame
         """
-        default_xda = self.load(self.get_default_band())
+        def_band = self.get_default_band()
+        default_xda = self.load(def_band)[
+            def_band
+        ]  # Forced to load as the nodata may not be positioned by default
         return rasters.get_footprint(default_xda)
 
     @abstractmethod
@@ -247,7 +266,7 @@ class Product:
         raise NotImplementedError("This method should be implemented by a child class")
 
     @abstractmethod
-    def crs(self) -> rcrs.CRS:
+    def crs(self) -> riocrs.CRS:
         """
         Get UTM projection of the tile
 
@@ -552,29 +571,34 @@ class Product:
             dict: Dictionary {band_name, band_xarray}
         """
         dem_bands = {}
-        dem_path = os.environ.get(DEM_PATH)
-        if not dem_path:
-            LOGGER.warning(
-                f"Dem path not set, unable to compute DEM bands ! "
-                f"Please set the environment variable {DEM_PATH}."
-            )
-        else:
-            for band in band_list:
-                assert is_dem(band)
-                if band == DEM:
-                    path = self._warp_dem(dem_path, resolution=resolution, size=size)
-                elif band == SLOPE:
-                    path = self._compute_slope(
-                        dem_path, resolution=resolution, size=size
-                    )
-                elif band == HILLSHADE:
-                    path = self._compute_hillshade(
-                        dem_path, resolution=resolution, size=size
-                    )
-                else:
-                    raise InvalidTypeError(f"Unknown DEM band: {band}")
+        if band_list:
+            dem_path = os.environ.get(DEM_PATH)
+            if not dem_path:
+                LOGGER.warning(
+                    f"Dem path not set, unable to compute DEM bands ! "
+                    f"Please set the environment variable {DEM_PATH}."
+                )
+            else:
+                for band in band_list:
+                    assert is_dem(band)
+                    if band == DEM:
+                        path = self._warp_dem(
+                            dem_path, resolution=resolution, size=size
+                        )
+                    elif band == SLOPE:
+                        path = self._compute_slope(
+                            dem_path, resolution=resolution, size=size
+                        )
+                    elif band == HILLSHADE:
+                        path = self._compute_hillshade(
+                            dem_path, resolution=resolution, size=size
+                        )
+                    else:
+                        raise InvalidTypeError(f"Unknown DEM band: {band}")
 
-                dem_bands[band] = rasters.read(path, resolution=resolution, size=size)
+                    dem_bands[band] = rasters.read(
+                        path, resolution=resolution, size=size
+                    )
 
         return dem_bands
 
