@@ -1,3 +1,19 @@
+# -*- coding: utf-8 -*-
+# Copyright 2021, SERTIT-ICube - France, https://sertit.unistra.fr/
+# This file is part of eoreader project
+#     https://github.com/sertit/eoreader
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 """ Super class for SAR products """
 import logging
 import os
@@ -101,6 +117,7 @@ class SarProduct(Product):
         self._band_folder = None
         self._snap_path = None
         self._raw_band_regex = None
+        self._snap_no_data = 0
 
         # Initialization from the super class
         super().__init__(product_path, archive_path, output_path)
@@ -116,6 +133,26 @@ class SarProduct(Product):
         self._set_sensor_mode()
         self.pol_channels = self._get_raw_bands()
 
+    def footprint(self) -> gpd.GeoDataFrame:
+        """
+        Get UTM footprint of the products (without nodata, *in french == emprise utile*)
+
+        ```python
+        >>> from eoreader.reader import Reader
+        >>> path = r"S2A_MSIL1C_20200824T110631_N0209_R137_T30TTK_20200824T150432.SAFE.zip"
+        >>> prod = Reader().open(path)
+        >>> prod.footprint()
+           index                                           geometry
+        0      0  POLYGON ((199980.000 4500000.000, 199980.000 4...
+        ```
+
+        Returns:
+            gpd.GeoDataFrame: Footprint as a GeoDataFrame
+        """
+        return rasters.get_footprint(
+            self.get_default_band_path()
+        )  # Processed by SNAP: the nodata is set
+
     def get_default_band(self) -> BandNames:
         """
         Get default band:
@@ -654,7 +691,7 @@ class SarProduct(Product):
         # Create target dir (tmp dir)
         with tempfile.TemporaryDirectory() as tmp_dir:
             # Out files
-            target_file = os.path.join(tmp_dir, f"{self.condensed_name}_DSPK")
+            target_file = os.path.join(tmp_dir, f"{self.condensed_name}")
             dspk_dim = target_file + ".dim"
 
             # Despeckle graph
@@ -681,11 +718,11 @@ class SarProduct(Product):
                 misc.run_cli(cmd_list)
 
             # Convert DIMAP images to GeoTiff
-            out = self._write_sar(dspk_dim, band.value)
+            out = self._write_sar(dspk_dim, band.value, dspk=True)
 
         return out
 
-    def _write_sar(self, dim_path: str, pol_up: str):
+    def _write_sar(self, dim_path: str, pol_up: str, dspk=False):
         """
         Write SAR image on disk.
 
@@ -703,13 +740,15 @@ class SarProduct(Product):
 
         # Open SAR image
         arr = rasters.read(img)
-        arr = rasters.set_nodata(arr, self.nodata)
+        arr = arr.where(arr != self._snap_no_data, np.nan)
 
         # Save the file as the terrain-corrected image
         file_path = os.path.join(
-            self.output, f"{files.get_filename(dim_path)}_{pol_up}.tif"
+            self.output,
+            f"{files.get_filename(dim_path)}_{pol_up}{'_DSPK' if dspk else ''}.tif",
         )
-        rasters.write(arr, file_path, dtype=np.float32)
+        # WARNING: Set nodata to 0 here as it is the value wanted by SNAP !
+        rasters.write(arr, file_path, dtype=np.float32, nodata=0)
 
         return file_path
 
